//
//  UndoableProperties.h
//  MailDemo
//
//  Created by David Phillip Oster on 3/17/07.
//  David Phillip Oster releases this, his work, under a creative commons license: 
//     share this, don't prevent others from sharing it, and you assume full liability
//
// This class implements a reusable Model base class that owns a
// NSMutableDictionary in a field called "properties"
//
// it handles undo, redo, and setting the text of the undo/redo items in the
// Edit menu.
//
// Use it by inheriting from it, and in the subclass
// 1.) in the init routine, pass the default dictionary, with the keys
//     that will be observed, to observeProperties:.
// 2.) provide a definition for -(NSUndoManager *)undoManager;
//     to give it access to an NSUndoManager
// 3.) Define appropriate keys in NSLocalizableStrings for undo items.
//     you need keys for New<Classname>, and for the dictionary keys.
//     If you don't provide strings, the keys themselves will be used.
//
// See Mailbox.m and Email.m for two examples of using this class

#import <Cocoa/Cocoa.h>


@interface UndoableProperties : NSObject {
  NSMutableDictionary * properties;
}
- (NSDictionary *) properties;
- (void)setProperties: (NSDictionary *)newProperties;

// subclasses should override
- (NSUndoManager *)undoManager;

// subclass should call in init
// Looks in LocalizableStrings for undo title for New<Subclass>
// example: "NewMailbox" = "create mailbox";
- (void)observeProperties:(NSDictionary *)defaultDict;

@end
