//
//  UndoableProperties.m
//  MailDemo
//
//  Created by David Phillip Oster on 3/17/07.
//  David Phillip Oster releases this, his work, under a creative commons license: 
//     share this, don't prevent others from sharing it, and you assume full liability
//

#import "UndoableProperties.h"


@implementation UndoableProperties

- (void)observeProperties:(NSDictionary *)defaultDict {
  [self setProperties:[[defaultDict mutableCopy] autorelease]];
  NSArray *keys = [defaultDict allKeys];
  NSEnumerator *enumer = [keys objectEnumerator];
  NSString *key;
  while (nil != (key = [enumer nextObject])) {
    NSString *keyPath = [@"properties." stringByAppendingString:key];
    [self addObserver:self forKeyPath:keyPath options:NSKeyValueObservingOptionOld context:NULL];
  }
  NSUndoManager *undo = [self undoManager];
  if ( ! ([undo isUndoing] || [undo isRedoing])) {
    NSString *undoString = [@"New" stringByAppendingString: NSStringFromClass([self class])];
    [undo setActionName:NSLocalizedString(undoString, @"")];
  }
}

- (void)unobserveAll:(NSArray *)keys {
  NSEnumerator *enumer = [keys objectEnumerator];
  NSString *key;
  while (nil != (key = [enumer nextObject])) {
    NSString *keyPath = [@"properties." stringByAppendingString:key];
    [self removeObserver:self forKeyPath:keyPath];
  }
}

- (void)setProperty:(id)value forKeyPath:(NSString *)keyPath {
  [self setValue:value forKeyPath:keyPath];
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
  NSUndoManager *undo = [self undoManager];
  id oldValue = [change objectForKey:NSKeyValueChangeOldKey];
  [[undo prepareWithInvocationTarget:object] setProperty:oldValue forKeyPath:keyPath];
  if ( ! ([undo isUndoing] || [undo isRedoing])) {
    NSString *undoString = [NSString stringWithFormat:NSLocalizedString(@"Change", @""), [keyPath pathExtension]];
    [undo setActionName:NSLocalizedString(undoString, @"")];
  }
}

- (NSMutableDictionary *) mutableProperties {
  return properties;
}

- (void) setMutableProperties: (NSMutableDictionary *)newProperties {
  if ([self properties] != newProperties){
    [ (UndoableProperties *) [[self undoManager] prepareWithInvocationTarget:self] setMutableProperties:[self mutableProperties]];
    [properties autorelease];
    properties = [newProperties mutableCopy];
  }
}

- (NSDictionary *) properties {
  return properties;
}

- (void) setProperties: (NSDictionary *)newProperties {
  if ([self properties] != newProperties){
    [self setMutableProperties:[[newProperties mutableCopy] autorelease]];
  }
}



// subclasses should override
- (NSUndoManager *)undoManager {
  return nil;
}

- (void) dealloc {
  [self unobserveAll:[properties allKeys]];
  [[self undoManager] removeAllActionsWithTarget:self];
  [properties release];  
  [super dealloc];
}

@end

